@@ -2,7 +2,8 @@ from enum import Enum
 from abc import ABC, abstractmethod
 from coopstructs.vectors import Vector2
 import math
-from typing import List
+from typing import List, Callable
+from coopstructs.toggles import EnumToggleable
 from coopstructs.geometry import Rectangle
 
 class Orientation(Enum):
@@ -21,11 +22,18 @@ class Orientation(Enum):
 
 
 class Curve(ABC):
-    def __init__(self, origin: Vector2):
+    def __init__(self, id: str, origin: Vector2):
         self.origin = origin
+        self.id = id
 
     def __str__(self):
-        return f"{type(self)} starting at {self.origin} and terminating at {self.end_point()}"
+        return f"{type(self)} starting at {self.origin} and terminating at {self.end_point()} [{self.id}]"
+
+    def __hash__(self):
+        return hash(self.id)
+
+    def __repr__(self):
+        return self.__str__()
 
     @abstractmethod
     def end_point(self):
@@ -35,10 +43,41 @@ class Curve(ABC):
     def line_representation(self) -> List:
         pass
 
+    @classmethod
+    def orientation(cls, origin, destination) -> List[Orientation]:
+        orientation = None
+        if origin.x == destination.x and origin.y > destination.y:
+            orientation = [Orientation.UP]
+        elif origin.x == destination.x and origin.y < destination.y:
+            orientation = [Orientation.DOWN]
+        elif origin.x < destination.x and origin.y == destination.y:
+            orientation = [Orientation.RIGHT]
+        elif origin.x > destination.x and origin.y == destination.y:
+            orientation = [Orientation.LEFT]
+        elif origin.x > destination.x and origin.y > destination.y:
+            orientation = [Orientation.UP_LEFT, Orientation.LEFT_UP]
+        elif origin.x > destination.x and origin.y < destination.y:
+            orientation = [Orientation.DOWN_LEFT, Orientation.LEFT_DOWN]
+        elif origin.x < destination.x and origin.y > destination.y:
+            orientation = [Orientation.UP_RIGHT, Orientation.RIGHT_UP]
+        elif origin.x < destination.x and origin.y < destination.y:
+            orientation = [Orientation.DOWN_RIGHT, Orientation.RIGHT_DOWN]
+        return orientation
+
+
 class Arc(Curve):
-    def __init__(self, curve_type: Orientation, origin: Vector2, arc_box_size: Vector2):
-        super().__init__(origin)
-        self.curve_type = curve_type
+
+    @classmethod
+    def from_arcbox(cls, id:str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
+        return Arc(id, orientation, origin, arc_box_size)
+
+    @classmethod
+    def from_origin_and_destination(cls, id:str, orientation: Orientation, origin: Vector2, destination: Vector2):
+        return Arc(id, orientation, origin, Vector2(destination.x - origin.x, destination.y - origin.y) * 2)
+
+    def __init__(self, id:str, orientation: Orientation, origin: Vector2, arc_box_size: Vector2):
+        super().__init__(id, origin)
+        self.orientation = orientation
         self.arc_box_size = arc_box_size
         self.arc_rad_start = None
         self.arc_rad_end = None
@@ -50,41 +89,41 @@ class Arc(Curve):
         return self.compute_curve_lines()
 
     def define_arc_radian_start_end(self):
-        if self.curve_type == Orientation.DOWN_LEFT:
+        if self.orientation == Orientation.DOWN_LEFT:
             self.arc_rad_start = 2 * math.pi
             self.arc_rad_end = 3 * math.pi / 2
-        elif self.curve_type == Orientation.DOWN_RIGHT:
+        elif self.orientation == Orientation.DOWN_RIGHT:
             self.arc_rad_start = math.pi
             self.arc_rad_end = 3 * math.pi / 2
-        elif self.curve_type == Orientation.LEFT_DOWN:
+        elif self.orientation == Orientation.LEFT_DOWN:
             self.arc_rad_start = math.pi / 2
             self.arc_rad_end = math.pi
-        elif self.curve_type == Orientation.LEFT_UP:
+        elif self.orientation == Orientation.LEFT_UP:
             self.arc_rad_start = 3 * math.pi / 2
             self.arc_rad_end = math.pi
-        elif self.curve_type == Orientation.UP_LEFT:
+        elif self.orientation == Orientation.UP_LEFT:
             self.arc_rad_start = 0
             self.arc_rad_end = math.pi / 2
-        elif self.curve_type == Orientation.UP_RIGHT:
+        elif self.orientation == Orientation.UP_RIGHT:
             self.arc_rad_start = math.pi
             self.arc_rad_end = math.pi / 2
-        elif self.curve_type == Orientation.RIGHT_DOWN:
+        elif self.orientation == Orientation.RIGHT_DOWN:
             self.arc_rad_start = math.pi / 2
             self.arc_rad_end = 0
-        elif self.curve_type == Orientation.RIGHT_UP:
+        elif self.orientation == Orientation.RIGHT_UP:
             self.arc_rad_start = 3 * math.pi / 2
             self.arc_rad_end = 2 * math.pi
         else:
-            raise Exception(f"Invalid CurveType: {self.curve_type} to draw")
+            raise Exception(f"Invalid curve orientation: [{self.orientation}]")
 
     def end_point(self):
-        if self.curve_type in (Orientation.DOWN_LEFT,  Orientation.LEFT_DOWN):
+        if self.orientation in (Orientation.DOWN_LEFT,  Orientation.LEFT_DOWN):
             return Vector2(int(self.origin.x - self.arc_box_size.x / 2), int(self.origin.y + self.arc_box_size.y / 2))
-        elif self.curve_type in (Orientation.LEFT_UP, Orientation.UP_LEFT):
+        elif self.orientation in (Orientation.LEFT_UP, Orientation.UP_LEFT):
             return Vector2(int(self.origin.x - self.arc_box_size.x / 2), int(self.origin.y - self.arc_box_size.y / 2))
-        elif self.curve_type in (Orientation.UP_RIGHT, Orientation.RIGHT_UP):
+        elif self.orientation in (Orientation.UP_RIGHT, Orientation.RIGHT_UP):
             return Vector2(int(self.origin.x + self.arc_box_size.x / 2), int(self.origin.y - self.arc_box_size.y / 2))
-        elif self.curve_type in (Orientation.RIGHT_DOWN, Orientation.DOWN_RIGHT):
+        elif self.orientation in (Orientation.RIGHT_DOWN, Orientation.DOWN_RIGHT):
             return Vector2(int(self.origin.x + self.arc_box_size.x / 2), int(self.origin.y + self.arc_box_size.y / 2))
         else:
             raise Exception("Incorrect Curve type with orientation")
@@ -101,7 +140,7 @@ class Arc(Curve):
             else:
                 start = Vector2(b_points[ii - 1].x, b_points[ii - 1].y)
                 end = Vector2(b_points[ii].x, b_points[ii].y)
-                ret.append(Line(start, end))
+                ret.append(Line(self.id + f"_{ii}", start, end))
         return ret
 
     def compute_curve_points(self, numPoints=None) -> List[Vector2]:
@@ -131,28 +170,28 @@ class Arc(Curve):
         return Vector2(int(x), int(y)) + rotation_point
 
     def arc_box(self):
-        if self.curve_type == Orientation.DOWN_LEFT:
+        if self.orientation == Orientation.DOWN_LEFT:
             return [self.origin.x - self.arc_box_size.x, self.origin.y - self.arc_box_size.y / 2,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.DOWN_RIGHT:
+        elif self.orientation == Orientation.DOWN_RIGHT:
             return [self.origin.x, self.origin.y - self.arc_box_size.y / 2,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.LEFT_DOWN:
+        elif self.orientation == Orientation.LEFT_DOWN:
             return [self.origin.x - self.arc_box_size.x / 2, self.origin.y,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.LEFT_UP:
+        elif self.orientation == Orientation.LEFT_UP:
             return [self.origin.x - self.arc_box_size.x / 2, self.origin.y - self.arc_box_size.y,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.UP_LEFT:
+        elif self.orientation == Orientation.UP_LEFT:
             return [self.origin.x - self.arc_box_size.x, self.origin.y - self.arc_box_size.y / 2,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.UP_RIGHT:
+        elif self.orientation == Orientation.UP_RIGHT:
             return [self.origin.x, self.origin.y - self.arc_box_size.y / 2,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.RIGHT_UP:
+        elif self.orientation == Orientation.RIGHT_UP:
             return [self.origin.x - self.arc_box_size.x / 2, self.origin.y - self.arc_box_size.y,
                     self.arc_box_size.x, self.arc_box_size.y]
-        elif self.curve_type == Orientation.RIGHT_DOWN:
+        elif self.orientation == Orientation.RIGHT_DOWN:
             return [self.origin.x - self.arc_box_size.x / 2, self.origin.y,
                     self.arc_box_size.x, self.arc_box_size.y]
         else:
@@ -164,31 +203,11 @@ class Arc(Curve):
 
 
 class Line(Curve):
-    def __init__(self, origin: Vector2, destination: Vector2):
-        super().__init__(origin)
+    def __init__(self, id:str, origin: Vector2, destination: Vector2):
+        super().__init__(id, origin)
         self.destination = destination
         self.length = self.length()
 
-    def orientation(self):
-        orientation = None
-        if self.origin.x == self.destination.x and self.origin.y > self.destination.y:
-            orientation = Orientation.UP
-        elif self.origin.x == self.destination.x and self.origin.y < self.destination.y:
-            orientation = Orientation.DOWN
-        elif self.origin.x < self.destination.x and self.origin.y == self.destination.y:
-            orientation = Orientation.RIGHT
-        elif self.origin.x > self.destination.x and self.origin.y == self.destination.y:
-            orientation = Orientation.LEFT
-        elif self.origin.x > self.destination.x and self.origin.y > self.destination.y:
-            orientation = Orientation.UP_LEFT
-        elif self.origin.x > self.destination.x and self.origin.y < self.destination.y:
-            orientation = Orientation.DOWN_LEFT
-        elif self.origin.x < self.destination.x and self.origin.y > self.destination.y:
-            orientation = Orientation.UP_RIGHT
-        elif self.origin.x < self.destination.x and self.origin.y < self.destination.y:
-            orientation = Orientation.DOWN_RIGHT
-        return orientation
-
     def line_representation(self):
         return [self]
 
@@ -204,9 +223,9 @@ class Line(Curve):
                   f"{e}")
             raise
 
-class Bezier(Curve):
-    def __init__(self, control_points: List[Vector2]):
-        Curve.__init__(self, control_points[0])
+class CubicBezier(Curve):
+    def __init__(self, id:str, control_points: List[Vector2]):
+        Curve.__init__(self, id, control_points[0])
         self.control_points = control_points
 
     def line_representation(self):
@@ -224,14 +243,14 @@ class Bezier(Curve):
             else:
                 start = Vector2(b_points[ii - 1][0], b_points[ii - 1][1])
                 end = Vector2(b_points[ii][0], b_points[ii][1])
-                ret.append(Line(start, end))
+                ret.append(Line(self.id + f"_{ii}", start, end))
         return ret
 
     def compute_bezier_points(self, vertices, numPoints=None):
         if numPoints is None:
             numPoints = 30
         if numPoints < 2 or len(vertices) != 4:
-            return None
+            return []
 
         result = []
 
@@ -290,3 +309,106 @@ class Bezier(Curve):
             result.append((int(pointX), int(pointY)))
 
         return result
+
+
+class CurveType(Enum):
+    ARC = 1,
+    CUBICBEZIER = 2,
+    LINE = 3,
+    CATMULLROM = 4
+
+
+class CurveBuilder:
+
+    def __init__(self, id_provider: Callable [[], str]):
+        self.curves = {} # {id: curve}
+        self.current_points = []
+        self.current_curve_type = EnumToggleable(CurveType)
+        self.current_orientation = EnumToggleable(Orientation)
+        self.id_provider = id_provider
+
+
+    def toggle_curve_type(self):
+        self.current_curve_type.toggle()
+        self.current_points = []
+
+    def save_curve(self):
+        new_curves = self.curves_from_points(self.current_curve_type.value, self.current_points)
+        for curve in new_curves:
+            self.curves[self.id_provider()] = curve
+        self.current_points = []
+
+    def add_point(self, point: Vector2):
+        self.current_points.append(point)
+
+    def remove_point(self):
+        self.current_points.pop()
+
+    def orientation(self, origin, destination) -> List[Orientation]:
+        orientation = None
+        if origin.x == destination.x and origin.y > destination.y:
+            orientation = [Orientation.UP]
+        elif origin.x == destination.x and origin.y < destination.y:
+            orientation = [Orientation.DOWN]
+        elif origin.x < destination.x and origin.y == destination.y:
+            orientation = [Orientation.RIGHT]
+        elif origin.x > destination.x and origin.y == destination.y:
+            orientation = [Orientation.LEFT]
+        elif origin.x > destination.x and origin.y > destination.y:
+            orientation = [Orientation.UP_LEFT, Orientation.LEFT_UP]
+        elif origin.x > destination.x and origin.y < destination.y:
+            orientation = [Orientation.DOWN_LEFT, Orientation.LEFT_DOWN]
+        elif origin.x < destination.x and origin.y > destination.y:
+            orientation = [Orientation.UP_RIGHT, Orientation.RIGHT_UP]
+        elif origin.x < destination.x and origin.y < destination.y:
+            orientation = [Orientation.DOWN_RIGHT, Orientation.RIGHT_DOWN]
+        return orientation
+
+    def curves_from_points(self, curve_type: CurveType, points: List[Vector2]) -> List[Curve]:
+        curves = []
+        if len(points) <2:
+            return curves
+
+        if curve_type == CurveType.LINE:
+            curves = self.lines_from_points(points)
+        elif curve_type == CurveType.ARC:
+            curves = self.arcs_from_points(points)
+        elif curve_type == CurveType.CUBICBEZIER:
+            curves = self.beziers_from_points(points)
+
+        return curves
+
+    def lines_from_points(self, points):
+        lines = []
+        for ii in range(len(points) - 1):
+            lines.append(Line(self.id_provider(), points[ii], points[ii + 1]))
+        return lines
+
+    def arcs_from_points(self, points):
+        curves = []
+        for ii in range(len(points) - 1):
+            orientation = self.orientation(points[ii], points[ii + 1])[0]
+            if orientation in (Orientation.UP, Orientation.RIGHT, Orientation.DOWN, Orientation.LEFT):
+                curves.append(Line(self.id_provider(), points[ii], points[ii + 1]))
+            else:
+                curves.append(Arc(self.id_provider(), orientation, points[0],
+                            Vector2(abs(points[ii + 1].x - points[ii].x) * 2, abs(points[ii + 1].y - points[ii].y) * 2))
+                              )
+        return curves
+
+    def beziers_from_points(self, points):
+        curves = []
+        for ii in range(0, len(points) - 4, 3):
+            curves.append(CubicBezier(self.id_provider(), points[ii:ii+3]))
+        return curves
+
+    def curves_from_temp_next_points(self, next_points: List[Vector2]):
+        all_points = self.current_points + next_points
+        points = []
+        for i in all_points:
+            if i not in points:
+                points.append(i)
+
+        return self.curves_from_points(self.current_curve_type.value, points)
+
+
