@@ -5,6 +5,8 @@ class Rectangle:
         self.height = height
         self.width = width
 
+    def points_tuple(self):
+        return ((self.x, self.y), (self.x + self.width, self.y), (self.x + self.width, self.y + self.height), (self.x, self.y + self.height))
 
     def __str__(self):
         return f"TopLeft: <{self.x}, {self.y}>, Size: H{self.height} x W{self.width}"
