@@ -213,6 +213,14 @@ class IVector:
         else:
             raise NotImplementedError(f"Unimplemented interpolation type: {interpolate_type}")
 
+    def absolute(self):
+
+        abs_coords = {}
+        for key, val in self.coords.items():
+            abs_coords[key] = abs(val)
+
+        return IVector(abs_coords)
+
 class Vector2 (IVector):
     def __init__(self, x: float, y: float):
         IVector.__init__(self)
